@@ -34,6 +34,7 @@
 extern gboolean help;
 guint errors=0;
 GList *ignore_errors_list=NULL;
+GHashTable *ignore_errors_set=NULL;
 GAsyncQueue *stream_queue = NULL;
 gboolean use_defer= FALSE;
 gboolean check_row_count= FALSE;
@@ -84,50 +85,6 @@ void free_set_names(){
   g_free(set_names_statement);
 }
 
-char *generic_checksum(MYSQL *conn, char *database, char *table, const gchar *query_template, int column_number){
-  char *query = g_strdup_printf(query_template, database, table);
-  struct M_ROW *mr = m_store_result_single_row( conn, query, "Error dumping checksum (%s.%s)", database, table);
-  g_free(query);
-  char * r=NULL;
-  if (mr->row)
-    r=g_strdup_printf("%s",mr->row[column_number]);
-  m_store_result_row_free(mr);
-  return r;
-}
-
-char * checksum_table(MYSQL *conn, char *database, char *table){
-  return generic_checksum(conn, database, table, "CHECKSUM TABLE `%s`.`%s`", 1);
-}
-
-
-char * checksum_table_structure(MYSQL *conn, char *database, char *table){
-  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(CONCAT_WS(column_name, ordinal_position, data_type)) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.columns WHERE table_schema='%s' AND table_name='%s';", 0);
-}
-
-char * checksum_process_structure(MYSQL *conn, char *database, char *table){
-  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(replace(ROUTINE_DEFINITION,' ','')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.routines WHERE ROUTINE_SCHEMA='%s' order by ROUTINE_TYPE,ROUTINE_NAME", 0);
-}
-
-char * checksum_trigger_structure(MYSQL *conn, char *database, char *table){
-  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(REPLACE(REPLACE(REPLACE(REPLACE(ACTION_STATEMENT, CHAR(32), ''), CHAR(13), ''), CHAR(10), ''), CHAR(9), '')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.triggers WHERE EVENT_OBJECT_SCHEMA='%s' AND EVENT_OBJECT_TABLE='%s';",0);
-}
-
-char * checksum_trigger_structure_from_database(MYSQL *conn, char *database, char *table){
-  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(REPLACE(REPLACE(REPLACE(REPLACE(ACTION_STATEMENT, CHAR(32), ''), CHAR(13), ''), CHAR(10), ''), CHAR(9), '')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.triggers WHERE EVENT_OBJECT_SCHEMA='%s';",0);
-}
-
-char * checksum_view_structure(MYSQL *conn, char *database, char *table){
-  return generic_checksum(conn, database, table,"SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(REPLACE(VIEW_DEFINITION,TABLE_SCHEMA,'')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.views WHERE TABLE_SCHEMA='%s' AND TABLE_NAME='%s';",0);
-}
-
-char * checksum_database_defaults(MYSQL *conn, char *database, char *table){
-  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(concat(DEFAULT_CHARACTER_SET_NAME,DEFAULT_COLLATION_NAME)) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.SCHEMATA WHERE SCHEMA_NAME='%s' ;",0);
-}
-
-char * checksum_table_indexes(MYSQL *conn, char *database, char *table){
-  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(CONCAT_WS(TABLE_NAME,INDEX_NAME,SEQ_IN_INDEX,COLUMN_NAME)) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.STATISTICS WHERE TABLE_SCHEMA='%s' AND TABLE_NAME='%s' ORDER BY INDEX_NAME,SEQ_IN_INDEX,COLUMN_NAME", 0);
-}
-
 GKeyFile * load_config_file(gchar * config_file){
   GError *error = NULL;
   GKeyFile *kf = g_key_file_new ();
@@ -303,6 +260,44 @@ void load_hash_of_all_variables_perproduct_from_key_file(GKeyFile *kf, GHashTabl
   load_hash_from_key_file(kf,set_session_hash,s->str);
 }
 
+// This function is like g_key_file_has_group but is case insensitive
+gboolean m_key_file_has_group (GKeyFile* kf, const gchar* group_name){
+  gchar **groups=g_key_file_get_groups(kf, NULL);
+  guint i=0;
+  while (groups[i]){
+    if (!g_ascii_strcasecmp(groups[i],group_name)){
+      g_strfreev(groups);
+      return TRUE;
+    }
+    i++;
+  }
+  g_strfreev(groups);
+  return FALSE;
+}
+
+void load_options_for_product_from_key_file(GKeyFile *kf, GOptionContext *context, const gchar *app, int product, int major, int secondary, int revision){
+  GString *group=g_string_sized_new(50);
+  g_string_append(group, app);
+  g_string_append(group, "_");
+  g_string_append(group, g_ascii_strdown(get_product_name(product),-1));
+  g_message("searching group group %s",group->str);
+  if (g_key_file_has_group(kf,group->str)){
+    g_message("group found %s",group->str);
+    parse_key_file_group(kf, context, group->str);
+  }
+  g_string_append_printf(group, "_%d", major);
+  if (g_key_file_has_group(kf,group->str)){
+    parse_key_file_group(kf, context, group->str);
+  }
+  g_string_append_printf(group, "_%d", secondary);
+  if (g_key_file_has_group(kf,group->str)){
+    parse_key_file_group(kf, context, group->str);
+  }
+  g_string_append_printf(group, "_%d", revision);
+  if (g_key_file_has_group(kf,group->str)){
+    parse_key_file_group(kf, context, group->str);
+  }
+}
 
 void free_hash_table(GHashTable * hash){
   GHashTableIter iter;
@@ -484,6 +479,13 @@ guint strcount(gchar *text){
   return i;
 }
 
+gchar * common_build_schema_table_filename(gchar *_directory, char *database, char *table, const char *suffix){
+  GString *filename = g_string_sized_new(128);
+  g_string_append_printf(filename, "%s.%s-%s.sql", database, table, suffix);
+  gchar *r = g_build_filename(_directory?_directory:filename->str, _directory?filename->str:NULL, NULL);
+  g_string_free(filename,TRUE);
+  return r;
+}
 
 gchar * remove_new_line(gchar *to){
   if (to==NULL)
@@ -501,17 +503,20 @@ gchar * remove_new_line(gchar *to){
   return to;
 }
 
-void m_remove0(gchar * directory, const gchar * filename){
+gboolean m_remove0(gchar * directory, const gchar * filename){
     gchar *path = g_build_filename(directory == NULL?"":directory, filename, NULL);
     g_message("Removing file: %s", path);
-    if (remove(path) < 0)
+    if (remove(path) < 0){
       g_warning("Remove failed: %s (%s)", path, strerror(errno));
+      return FALSE;
+    }
     g_free(path);
+    return TRUE;
 }
 
 gboolean m_remove(gchar * directory, const gchar * filename){
   if (stream && no_delete == FALSE){
-    m_remove0(directory,filename);
+    return m_remove0(directory,filename);
   }
   return TRUE;
 }
@@ -614,7 +619,7 @@ void initialize_common_options(GOptionContext *context, const gchar *group){
     }
   }else{
     if (defaults_file == NULL){
-      g_message("Using no configuration file");
+      trace("Using no configuration file");
       return;
     }
   }
@@ -624,7 +629,7 @@ void initialize_common_options(GOptionContext *context, const gchar *group){
     defaults_extra_file=NULL;
   }
 
-//  g_message("Using default file: %s %s", defaults_file, defaults_extra_file);
+  trace("Using default file: %s %s", defaults_file, defaults_extra_file);
 
   gchar *new_defaults_file=NULL;
   if (!g_path_is_absolute(defaults_file)){
@@ -659,7 +664,7 @@ void initialize_common_options(GOptionContext *context, const gchar *group){
 
   if (extra_key_file!=NULL){ 
     if (g_key_file_has_group(extra_key_file, group )){
-      g_message("Parsing extra key file");
+      trace("Parsing extra key file");
       parse_key_file_group(extra_key_file, context, group);
       set_connection_defaults_file_and_group(defaults_extra_file, group);
     } 
@@ -669,7 +674,7 @@ void initialize_common_options(GOptionContext *context, const gchar *group){
   }else
     set_connection_defaults_file_and_group(defaults_extra_file, NULL);
 
-  g_message("Merging config files user: ");
+  trace("Merging config files user");
 
   m_key_file_merge(key_file, extra_key_file);
 
@@ -880,14 +885,26 @@ GRecMutex * g_rec_mutex_new(){
 */
 gboolean read_data(FILE *file, GString *data,
                    gboolean *eof, guint *line) {
-  char buffer[4096];
+
+  // Perf: Larger buffer reduces syscalls (16KB vs 4KB)
+  char buffer[65536];
   size_t l;
 
   while (fgets(buffer, sizeof(buffer), file)) {
-    l= strlen(buffer);
-    //g_assert(l > 0 && l < sizeof(buffer));
-    g_string_append(data, buffer);
-    if (buffer[l - 1] == '\n') {
+    // Perf: Use memchr to find newline - SIMD optimized, O(n/16) vs O(n)
+    // This is faster than strlen() which must scan to NUL terminator
+    char *newline = memchr(buffer, '\n', sizeof(buffer));
+
+    if (newline) {
+      // Found newline - calculate exact length
+      l = newline - buffer + 1;
+    } else {
+      // No newline found - use strlen for partial line
+      l = strlen(buffer);
+    }
+    // Perf: Use g_string_append_len with known length (avoids strlen inside append)
+    g_string_append_len(data, buffer, l);
+    if (newline) {
       (*line)++;
       *eof= FALSE;
       return TRUE;
@@ -1237,12 +1254,14 @@ gboolean str_list_has_str(gchar ** str_list, const gchar* str){
 }
 
 void parse_object_to_export(struct object_to_export *object_to_export,gchar *val){
-  if (!val){
-    object_to_export->no_data=FALSE;
-    object_to_export->no_schema=FALSE;
-    object_to_export->no_trigger=FALSE;
+  object_to_export->no_data=FALSE;
+  object_to_export->no_schema=FALSE;
+  object_to_export->no_view=FALSE;
+  object_to_export->no_trigger=FALSE;
+  object_to_export->no_index=FALSE;
+  object_to_export->no_constraint=FALSE;
+  if (!val)
     return;
-  }
   gchar **split_option = g_strsplit(val, ",", 4);
   object_to_export->no_data=!str_list_has_str(split_option,"DATA");
   object_to_export->no_schema=!str_list_has_str(split_option,"SCHEMA");
@@ -1250,11 +1269,17 @@ void parse_object_to_export(struct object_to_export *object_to_export,gchar *val
   if (str_list_has_str(split_option,"ALL")){
     object_to_export->no_data=FALSE;
     object_to_export->no_schema=FALSE;
+    object_to_export->no_view=FALSE;
+    object_to_export->no_index=FALSE;
+    object_to_export->no_constraint=FALSE;
     object_to_export->no_trigger=FALSE;
   }
   if (str_list_has_str(split_option,"NONE")){
     object_to_export->no_data=TRUE;
     object_to_export->no_schema=TRUE;
+    object_to_export->no_view=TRUE;
+    object_to_export->no_index=TRUE;
+    object_to_export->no_constraint=TRUE;
     object_to_export->no_trigger=TRUE;
   }
   g_strfreev(split_option);
@@ -1329,10 +1354,15 @@ void discard_mysql_output(MYSQL *conn){
   }
 }
 
+gboolean should_ignore_error_code(guint error_code) {
+  if (ignore_errors_set == NULL) return FALSE;
+  return g_hash_table_contains(ignore_errors_set, GINT_TO_POINTER(error_code));
+}
+
 static void m_log(MYSQL *conn, void log_fun_1(const char *, ...), void log_fun_2(const char *, ...), const char *fmt, va_list args){
   if (fmt && log_fun_1){
     gchar *c=g_strdup_vprintf(fmt,args);
-    if (log_fun_2 && g_list_find(ignore_errors_list, GINT_TO_POINTER(mysql_errno(conn))))
+    if (log_fun_2 && should_ignore_error_code(mysql_errno(conn)))
       log_fun_2("%s - ERROR %d: %s",c, mysql_errno(conn), mysql_error(conn));
     else{
       if (mysql_errno(conn)){
@@ -1357,40 +1387,44 @@ static gboolean m_queryv(  MYSQL *conn, const gchar *query, void log_fun_1(const
 
 gboolean m_query(  MYSQL *conn, const gchar *query, void log_fun(const char *, ...) , const char *fmt, ...){
   va_list args;
-  if (fmt)
-    va_start(args, fmt);
-  return m_queryv(conn, query, log_fun, NULL, fmt,args);
+  va_start(args, fmt);
+  gboolean result = m_queryv(conn, query, log_fun, NULL, fmt, args);
+  va_end(args);
+  return result;
 }
 
 // Executes the query, if there is an error it send critical stopping the process unless the error is ignored
 gboolean m_query_warning(  MYSQL *conn, const gchar *query, const char *fmt, ...){
   va_list args;
-  if (fmt)
-    va_start(args, fmt);
-  return m_queryv(conn, query, m_warning, NULL, fmt,args);
+  va_start(args, fmt);
+  gboolean result = m_queryv(conn, query, m_warning, NULL, fmt, args);
+  va_end(args);
+  return result;
 }
 
 // Executes the query, if there is an error it send critical stopping the process unless the error is ignored
 gboolean m_query_critical(  MYSQL *conn, const gchar *query, const char *fmt, ...){
   va_list args;
-  if (fmt)
-    va_start(args, fmt);
-  return m_queryv(conn, query, m_critical, m_warning, fmt,args);
+  va_start(args, fmt);
+  gboolean result = m_queryv(conn, query, m_critical, m_warning, fmt, args);
+  va_end(args);
+  return result;
 }
 
 
 gboolean m_query_ext(  MYSQL *conn, const gchar *query, void log_fun_1(const char *, ...), void log_fun_2(const char *, ...), const char *fmt, ...){
   va_list args;
-  if (fmt)
-    va_start(args, fmt);
-  return m_queryv(conn, query, log_fun_1, log_fun_2, fmt,args);
+  va_start(args, fmt);
+  gboolean result = m_queryv(conn, query, log_fun_1, log_fun_2, fmt, args);
+  va_end(args);
+  return result;
 }
 
 gboolean m_query_verbose(MYSQL *conn, const char *q, void log_fun(const char *, ...) , const char *fmt, ...){
   va_list args;
-  if (fmt)
-    va_start(args, fmt);
-  gboolean res= m_queryv(conn, q, log_fun, NULL, fmt, args);
+  va_start(args, fmt);
+  gboolean res = m_queryv(conn, q, log_fun, NULL, fmt, args);
+  va_end(args);
   if (!res)
     g_message("%s: OK", q);
   return res;
@@ -1408,32 +1442,35 @@ MYSQL_RES *m_resultv(MYSQL_RES * m_result(MYSQL *), MYSQL *conn, const gchar *qu
 
 MYSQL_RES *m_store_result_critical(MYSQL *conn, const gchar *query, const char *fmt, ...){
   va_list args;
-  if (fmt)
-    va_start(args, fmt);
-  return m_resultv(mysql_store_result, conn, query, m_critical, m_warning, fmt, args);
+  va_start(args, fmt);
+  MYSQL_RES *result = m_resultv(mysql_store_result, conn, query, m_critical, m_warning, fmt, args);
+  va_end(args);
+  return result;
 }
 
 MYSQL_RES *m_store_result(MYSQL *conn, const gchar *query, void log_fun(const char *, ...) , const char *fmt, ...){
   va_list args;
-  if (fmt)
-    va_start(args, fmt);
-  return m_resultv(mysql_store_result, conn, query, log_fun, NULL, fmt, args);
+  va_start(args, fmt);
+  MYSQL_RES *result = m_resultv(mysql_store_result, conn, query, log_fun, NULL, fmt, args);
+  va_end(args);
+  return result;
 }
 
 MYSQL_RES *m_use_result(MYSQL *conn, const gchar *query, void log_fun(const char *, ...) , const char *fmt, ...){
   va_list args;
-  if (fmt)
-    va_start(args, fmt);
-  return m_resultv(mysql_use_result, conn, query, log_fun, NULL, fmt, args);
+  va_start(args, fmt);
+  MYSQL_RES *result = m_resultv(mysql_use_result, conn, query, log_fun, NULL, fmt, args);
+  va_end(args);
+  return result;
 }
 
 struct M_ROW* m_store_result_row(MYSQL *conn, const gchar *query, void log_fun_1(const char *, ...), void log_fun_2(const char *, ...), const char *fmt, ...){
   va_list args;
-  if (fmt)
-    va_start(args, fmt);
+  va_start(args, fmt);
   struct M_ROW *mr=g_new0(struct M_ROW,1);
   mr->row=NULL;
   mr->res = m_resultv(mysql_store_result, conn, query, log_fun_1, log_fun_2, fmt, args);
+  va_end(args);
   if (mr->res)
     mr->row= mysql_fetch_row(mr->res);
   return mr;
@@ -1441,8 +1478,7 @@ struct M_ROW* m_store_result_row(MYSQL *conn, const gchar *query, void log_fun_1
 
 struct M_ROW* m_store_result_single_row(MYSQL *conn, const gchar *query, const char *fmt, ...){
   va_list args;
-  if (fmt)
-    va_start(args, fmt);
+  va_start(args, fmt);
   struct M_ROW *mr=g_new0(struct M_ROW,1);
   mr->row=NULL;
   mr->res = m_resultv(mysql_store_result, conn, query, m_critical, m_warning, fmt, args);
@@ -1452,6 +1488,7 @@ struct M_ROW* m_store_result_single_row(MYSQL *conn, const gchar *query, const c
     if (!mr->row)
       m_log(conn, m_critical, m_warning, fmt, args);
   }
+  va_end(args);
   return mr;
 }
 
