@@ -56,6 +56,7 @@
 #define BINARY_CHARSET "binary"
 #define AUTO_CHARSET "auto"
 extern GList *ignore_errors_list;
+extern GHashTable *ignore_errors_set;
 extern const gchar *start_replica;
 extern const gchar *stop_replica;
 extern const gchar *start_replica_sql_thread;
@@ -75,7 +76,10 @@ void initialize_gzip_cmd();
 struct object_to_export{
   gboolean no_data;
   gboolean no_schema;
+  gboolean no_view;
   gboolean no_trigger;
+  gboolean no_index;
+  gboolean no_constraint;
 };
 
 struct configuration_per_table{
@@ -121,17 +125,9 @@ struct function_pointer{
 };
 
 gchar * remove_new_line(gchar *to);
-char * checksum_table_structure(MYSQL *conn, char *database, char *table);
-char * checksum_table(MYSQL *conn, char *database, char *table);
-char * checksum_process_structure(MYSQL *conn, char *database, char *table);
-char * checksum_trigger_structure(MYSQL *conn, char *database, char *table);
-char * checksum_trigger_structure_from_database(MYSQL *conn, char *database, char *table);
-char * checksum_view_structure(MYSQL *conn, char *database, char *table);
-char * checksum_database_defaults(MYSQL *conn, char *database, char *table);
-char * checksum_table_indexes(MYSQL *conn, char *database, char *table);
 int write_file(FILE * file, char * buff, int len);
 guint strcount(gchar *text);
-void m_remove0(gchar * directory, const gchar * filename);
+gboolean m_remove0(gchar * directory, const gchar * filename);
 gboolean m_remove(gchar * directory, const gchar * filename);
 GKeyFile * load_config_file(gchar * config_file);
 void load_config_group(GKeyFile *kf, GOptionContext *context, const gchar * group);
@@ -174,6 +170,7 @@ void m_message(const char *fmt, ...);
 void load_hash_of_all_variables_perproduct_from_key_file(GKeyFile *kf, GHashTable * set_session_hash, const gchar *str);
 GRecMutex * g_rec_mutex_new();
 gboolean read_data(FILE *file, GString *data, gboolean *eof, guint *line);
+gboolean should_ignore_error_code(guint error_code);
 gchar *m_date_time_new_now_local();
 
 void print_int(const char*_key, int val);
@@ -232,3 +229,5 @@ GThread * m_thread_new(const gchar* title, GThreadFunc func, gpointer data, cons
 void *monitor_throttling_thread (void *queue);
 gchar *set_names_statement_template(gchar *_set_names);
 void execute_set_names(MYSQL *conn, gchar *_set_names);
+gchar * common_build_schema_table_filename(gchar *_directory, char *database, char *table, const char *suffix);
+void load_options_for_product_from_key_file(GKeyFile *kf, GOptionContext *context, const gchar *app, int product, int major, int secondary, int revision);
