@@ -334,7 +334,7 @@ struct chunk_step_item *get_next_integer_chunk(struct db_table *dbt){
         goto end;
       }
       if (is_last_step(csi)){
-        trace("Last chunk on step in `%s`.`%s` assigned", dbt->database->name, dbt->table);
+        trace("Last chunk on step in `%s`.`%s` assigned", dbt->database->source_database, dbt->table);
         csi->status=UNSPLITTABLE;
         csi->deep=csi->deep+1;
         new_csi=clone_chunk_step_item(csi);
@@ -443,7 +443,7 @@ gboolean refresh_integer_min_max(MYSQL *conn, struct db_table *dbt, struct chunk
                         "SELECT %s MIN(%s%s%s),MAX(%s%s%s) FROM %s%s%s.%s%s%s%s%s",
                         is_mysql_like() ? "/*!40001 SQL_NO_CACHE */": "",
                         identifier_quote_character_str, csi->field, identifier_quote_character_str, identifier_quote_character_str, csi->field, identifier_quote_character_str,
-                        identifier_quote_character_str, dbt->database->name, identifier_quote_character_str, identifier_quote_character_str, dbt->table, identifier_quote_character_str,
+                        identifier_quote_character_str, dbt->database->source_database, identifier_quote_character_str, identifier_quote_character_str, dbt->table, identifier_quote_character_str,
                         csi->prefix?" WHERE ":"", csi->prefix?csi->prefix->str:""), NULL, "Query to get a new min and max failed", NULL);
   trace("refresh_integer_min_max: %s", query);
   g_free(query);
@@ -485,7 +485,7 @@ gboolean update_integer_min(MYSQL *conn, struct db_table *dbt, struct chunk_step
                         "SELECT %s %s%s%s FROM %s%s%s.%s%s%s WHERE %s ORDER BY %s%s%s ASC LIMIT 1",
                         is_mysql_like() ? "/*!40001 SQL_NO_CACHE */": "",
                         identifier_quote_character_str, csi->field, identifier_quote_character_str,
-                        identifier_quote_character_str, dbt->database->name, identifier_quote_character_str, identifier_quote_character_str, dbt->table, identifier_quote_character_str,
+                        identifier_quote_character_str, dbt->database->source_database, identifier_quote_character_str, identifier_quote_character_str, dbt->table, identifier_quote_character_str,
                         where->str, 
                         identifier_quote_character_str, csi->field, identifier_quote_character_str), NULL, "Query to get a new min failed", NULL);
   g_string_free(where,TRUE);
@@ -524,7 +524,7 @@ gboolean update_integer_max(MYSQL *conn,struct db_table *dbt, struct chunk_step_
                         "SELECT %s %s%s%s FROM %s%s%s.%s%s%s WHERE %s ORDER BY %s%s%s DESC LIMIT 1",
                         is_mysql_like() ? "/*!40001 SQL_NO_CACHE */": "",
                         identifier_quote_character_str, csi->field, identifier_quote_character_str,
-                        identifier_quote_character_str, dbt->database->name, identifier_quote_character_str, identifier_quote_character_str, dbt->table, identifier_quote_character_str,
+                        identifier_quote_character_str, dbt->database->source_database, identifier_quote_character_str, identifier_quote_character_str, dbt->table, identifier_quote_character_str,
                         where->str,
                         identifier_quote_character_str, csi->field, identifier_quote_character_str), NULL, "Query to get a new max failed", NULL);
   g_string_free(where,TRUE);
@@ -762,18 +762,18 @@ retry:
       close_files(tj);
       write_table_job_into_file(tj);
     }else if (is_last(csi)) {
-      trace("Thread %d: I-Chunk 3: Last chunk on `%s`.`%s` no need to calculate anything else after finish", td->thread_id, tj->dbt->database->name, tj->dbt->table);
+      trace("Thread %d: I-Chunk 3: Last chunk on `%s`.`%s` no need to calculate anything else after finish", td->thread_id, tj->dbt->database->source_database, tj->dbt->table);
       write_table_job_into_file(tj);
     }else{
-      GDateTime *from = g_date_time_new_now_local();
+      // Perf: Use g_get_monotonic_time() instead of GDateTime to eliminate allocations
+      // Both return microseconds - direct subtraction works
+      gint64 from_time = g_get_monotonic_time();
       write_table_job_into_file(tj);
-      GDateTime *to = g_date_time_new_now_local();
+      gint64 to_time = g_get_monotonic_time();
 
 // Step 3.1: Updating Step length
 
-      GTimeSpan diff=g_date_time_difference(to,from);
-      g_date_time_unref(from);
-      g_date_time_unref(to);
+      GTimeSpan diff = to_time - from_time;  // Zero allocation, same precision
       g_mutex_lock(csi->mutex);
 
 
@@ -793,7 +793,7 @@ retry:
 
       if (diff>0 && tj->num_rows_of_last_run>0){
         cs->integer_step.step=tj->num_rows_of_last_run*max_time_per_select*G_TIME_SPAN_SECOND/diff;
-        trace("Thread %d: I-Chunk 3: Step size on `%s`.`%s` is %ld  ( %ld %ld)", td->thread_id, tj->dbt->database->name, tj->dbt->table, cs->integer_step.step, tj->num_rows_of_last_run, diff);
+        trace("Thread %d: I-Chunk 3: Step size on `%s`.`%s` is %ld  ( %ld %ld)", td->thread_id, tj->dbt->database->source_database, tj->dbt->table, cs->integer_step.step, tj->num_rows_of_last_run, diff);
       }else{
         cs->integer_step.step*=2;
         cs->integer_step.check_min=TRUE;
