@@ -36,7 +36,7 @@
 #include "myloader_restore.h"
 #include "myloader_pmm.h"
 #include "myloader_restore_job.h"
-#include "myloader_intermediate_queue.h"
+#include "myloader_process_filename.h"
 #include "myloader_arguments.h"
 #include "myloader_global.h"
 #include "myloader_worker_index.h"
@@ -44,6 +44,8 @@
 #include "myloader_worker_loader.h"
 #include "myloader_worker_post.h"
 #include "myloader_control_job.h"
+#include "myloader_database.h"
+#include "myloader_worker_loader_main.h"
 
 guint commit_count = 1000;
 gchar *input_directory = NULL;
@@ -58,7 +60,7 @@ gboolean optimize_keys_all_tables = FALSE;
 gboolean kill_at_once = FALSE;
 gboolean enable_binlog = FALSE;
 gboolean disable_redo_log = FALSE;
-enum checksum_modes checksum_mode= CHECKSUM_FAIL;
+enum checksum_modes checksum_mode= CHECKSUM_WARN;  // Issue #1975: Warn by default instead of fail
 gboolean skip_triggers = FALSE;
 gboolean skip_constraints = FALSE;
 gboolean skip_indexes = FALSE;
@@ -66,13 +68,14 @@ gboolean skip_post = FALSE;
 gboolean serial_tbl_creation = FALSE;
 gboolean resume = FALSE;
 guint rows = 0;
+guint num_sequences = 0;
 guint sequences = 0;
 guint sequences_processed = 0;
 GMutex sequences_mutex;
 gchar *source_db = NULL;
 extern guint errors;
 guint max_errors= 0;
-struct restore_errors detailed_errors = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+struct restore_errors detailed_errors = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
 guint max_threads_for_schema_creation=4;
 guint max_threads_for_index_creation=4;
 guint max_threads_for_post_creation= 1;
@@ -80,7 +83,7 @@ guint retry_count= 10;
 gboolean stream = FALSE;
 gboolean no_delete = FALSE;
 
-extern GHashTable *db_hash;
+extern GHashTable *database_hash;
 extern gboolean shutdown_triggered;
 extern gboolean skip_definer;
 extern gboolean local_infile;
@@ -178,29 +181,6 @@ void show_dbt(void* _key, void* dbt, void *total){
  //        //*((guint *)total) + 2+ ((struct db_table*)dbt)->schema_state >= CREATED /*ALL_DONE*/ ? 1 : 0;
   }
 
-void create_database(struct thread_data *td, gchar *database) {
-
-  const gchar *filename =
-      g_strdup_printf("%s-schema-create.sql%s", database, exec_per_thread_extension);
-  const gchar *filepath = g_strdup_printf("%s/%s-schema-create.sql%s",
-                                            directory, database, exec_per_thread_extension);
-
-  if (drop_database)
-    execute_drop_database(td, database);
-  if (g_file_test(filepath, G_FILE_TEST_EXISTS)) {
-    g_atomic_int_add(&(detailed_errors.schema_errors), restore_data_from_mydumper_file(td, filename, TRUE, NULL));
-  } else {
-    GString *data = g_string_new("CREATE DATABASE IF NOT EXISTS ");
-    g_string_append_printf(data,"`%s`", database);
-    trace("Creating schema %s", database);
-    if (restore_data_in_gstring_extended(td, data , TRUE, NULL, m_critical, "Failed to create database: %s", database) )
-      g_atomic_int_inc(&(detailed_errors.schema_errors));
-    g_string_free(data, TRUE);
-  }
-
-  return;
-}
-
 void print_help(){
     print_string("host", hostname);
     print_string("user", username);
@@ -274,7 +254,7 @@ void print_help(){
     print_string("directory",input_directory);
     print_string("logfile",logfile);
 
-    print_string("database",db);
+    print_string("database",target_db);
     print_string("quote-character",identifier_quote_character_str);
     print_bool("resume",resume);
     print_int("threads",num_threads);
@@ -292,12 +272,14 @@ void print_errors(){
   if (detailed_errors.tablespace_errors ||
     detailed_errors.schema_errors ||
     detailed_errors.data_errors ||
+    detailed_errors.data_warnings ||
     detailed_errors.view_errors ||
     detailed_errors.sequence_errors ||
     detailed_errors.index_errors ||
     detailed_errors.trigger_errors ||
     detailed_errors.constraints_errors ||
     detailed_errors.post_errors ||
+    detailed_errors.skip_errors ||
     detailed_errors.retries)
     g_message(
     "Errors found:\n"
@@ -312,6 +294,7 @@ void print_errors(){
     "- Post:      \t%d\n"
     "Warnings found:\n"
     "- Data:\t%d\n"
+    "- Skip Schema:\t%d\n"
     "Retries:\t%d",
     detailed_errors.tablespace_errors,
     detailed_errors.schema_errors,
@@ -323,13 +306,14 @@ void print_errors(){
     detailed_errors.constraints_errors,
     detailed_errors.post_errors,
     detailed_errors.data_warnings,
+    detailed_errors.skip_errors,
     detailed_errors.retries);
 
 
 }
 
 int main(int argc, char *argv[]) {
-  struct configuration conf = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL};
+  struct configuration conf = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL};
 
   GError *error = NULL;
   GOptionContext *context;
@@ -366,13 +350,25 @@ int main(int argc, char *argv[]) {
 
   g_strfreev(tmpargv);
 
-  if (db == NULL && source_db != NULL) {
-    db = g_strdup(source_db);
+  if (target_db == NULL && source_db != NULL) {
+    target_db = g_strdup(source_db);
   }
 
   if (overwrite_unsafe)
     overwrite_tables= TRUE;
 
+  // Auto-scale schema/index threads based on CPU count
+  // Only auto-scale if user hasn't explicitly set these values (default is 4)
+  guint cpu_count = g_get_num_processors();
+  if (max_threads_for_schema_creation == 4 && cpu_count > 4) {
+    guint auto_schema_threads = cpu_count > 8 ? 8 : cpu_count;
+    max_threads_for_schema_creation = auto_schema_threads;
+  }
+  if (max_threads_for_index_creation == 4 && cpu_count > 4) {
+    guint auto_index_threads = cpu_count > 8 ? 8 : cpu_count;
+    max_threads_for_index_creation = auto_index_threads;
+  }
+
   check_num_threads();
 
   if (num_threads > max_threads_per_table)
@@ -432,6 +428,8 @@ int main(int argc, char *argv[]) {
   if (tables_skiplist_file)
     read_tables_skiplist(tables_skiplist_file, &errors);
   initialize_process(&conf);
+  initialize_table(&conf);
+  initialize_database();
   initialize_common();
   initialize_connection(MYLOADER);
   initialize_regex(NULL);
@@ -466,6 +464,7 @@ int main(int argc, char *argv[]) {
   conf.post_table_queue = g_async_queue_new();
   conf.post_queue = g_async_queue_new();
   conf.index_queue = g_async_queue_new();
+  conf.ready_table_queue = g_async_queue_new();
   conf.view_queue = g_async_queue_new();
   conf.ready = g_async_queue_new();
   conf.pause_resume = g_async_queue_new();
@@ -497,7 +496,7 @@ int main(int argc, char *argv[]) {
   /* TODO: if conf is singleton it must be accessed as global variable */
   initialize_worker_schema(&conf);
   initialize_worker_index(&conf);
-  initialize_intermediate_queue(&conf);
+  initialize_process_filename(&conf);
 
   if (stream){
     if (resume){
@@ -535,12 +534,9 @@ int main(int argc, char *argv[]) {
       m_error("Disabling redologs is not supported for version %d.%d.%d", get_major(), get_secondary(), get_revision());
     }
   }
-
-  if (database_db){
-    if (!no_schemas)
-      create_database(t, database_db->real_database);
-    database_db->schema_state=CREATED;
-  }
+  g_message("start_database");
+  start_database(t);
+  g_message("start_worker_schema");
   start_worker_schema();
   initialize_loader_threads(&conf);
 
@@ -560,9 +556,9 @@ int main(int argc, char *argv[]) {
     tl=tl->next;
   }
 
-  wait_schema_worker_to_finish();
-  wait_loader_threads_to_finish();
-  wait_control_job();
+  wait_schema_worker_to_finish(&conf);
+  wait_worker_loader_main();
+  enqueue_indexes_if_possible(&conf);
   create_index_shutdown_job(&conf);
   wait_index_worker_to_finish();
   initialize_post_loding_threads(&conf);
@@ -587,17 +583,20 @@ int main(int argc, char *argv[]) {
   if (checksum_mode != CHECKSUM_SKIP) {
     GHashTableIter iter;
     gchar *lkey;
-    g_hash_table_iter_init(&iter, db_hash);
+    g_hash_table_iter_init(&iter, database_hash);
     struct database *d= NULL;
     while (g_hash_table_iter_next(&iter, (gpointer *) &lkey, (gpointer *) &d)) {
       if (d->schema_checksum != NULL && !no_schemas)
-        checksum_ok&=checksum_database_template(d->real_database, d->schema_checksum, conn,
+        checksum_ok&=checksum_database_template(d->target_database, d->schema_checksum, conn,
                                   "Schema create checksum", checksum_database_defaults);
       if (d->post_checksum != NULL && !skip_post)
-        checksum_ok&=checksum_database_template(d->real_database, d->post_checksum, conn,
+        checksum_ok&=checksum_database_template(d->target_database, d->post_checksum, conn,
                                   "Post checksum", checksum_process_structure);
+      if (d->events_checksum != NULL && !skip_post)
+        checksum_ok&=checksum_database_template(d->target_database, d->events_checksum, conn,
+                                  "Events checksum", checksum_events_structure_from_database);
       if (d->triggers_checksum != NULL && !skip_triggers)
-        checksum_ok&=checksum_database_template(d->real_database, d->triggers_checksum, conn,
+        checksum_ok&=checksum_database_template(d->target_database, d->triggers_checksum, conn,
                                   "Triggers checksum", checksum_trigger_structure_from_database);
     }
   }
@@ -670,7 +669,7 @@ int main(int argc, char *argv[]) {
     struct db_table * dbt=tl->data;
     GTimeSpan diff1=g_date_time_difference(dbt->start_index_time,dbt->start_time);
     GTimeSpan diff2=g_date_time_difference(dbt->finish_time,dbt->start_index_time);
-    g_message("%s\t| %s\t| %s\t| `%s`.`%s`",print_time(diff1),print_time(diff2),print_time(diff1+diff2),dbt->real_database,dbt->real_table);
+    g_message("%s\t| %s\t| %s\t| `%s`.`%s`",print_time(diff1),print_time(diff2),print_time(diff1+diff2),dbt->target_database,dbt->source_table_name);
     tl=tl->next;
   }
 */
