@@ -17,16 +17,6 @@
 
 */
 
-
-#define SEQUENCE "sequence"
-#define TRIGGER "trigger"
-#define POST "post"
-#define TABLESPACE "tablespace"
-#define CREATE_DATABASE "create database"
-#define VIEW "view"
-#define INDEXES "indexes"
-#define CONSTRAINTS "constraints"
-
 extern struct restore_errors detailed_errors;
 extern gboolean program_version;
 extern guint verbose;
@@ -55,7 +45,6 @@ extern gboolean append_if_not_exist;
 extern gboolean optimize_keys;
 extern gboolean optimize_keys_all_tables;
 extern gboolean optimize_keys_per_table;
-extern gboolean intermediate_queue_ended;
 extern gboolean no_data;
 extern gboolean no_schemas;
 extern gboolean no_delete;
@@ -71,13 +60,13 @@ extern gboolean skip_constraints;
 extern gboolean skip_indexes;
 extern gboolean stream;
 extern gchar *compress_extension;
-extern gchar *db;
+extern gchar *target_db;
 extern gchar *directory;
 extern gchar *pmm_path;
 extern gchar *pmm_resolution ;
 extern gchar *source_db;
 extern gchar *tables_skiplist_file;
-extern GHashTable *db_hash;
+extern GHashTable *database_hash;
 extern GHashTable *tbl_hash;
 extern GString *set_session;
 extern guint commit_count;
@@ -90,6 +79,7 @@ extern guint max_threads_per_table;
 extern guint retry_count;
 extern guint num_threads;
 extern guint rows;
+extern guint num_sequences;
 extern guint sequences;
 extern guint sequences_processed;
 extern GMutex sequences_mutex;
@@ -103,7 +93,6 @@ extern gchar **exec_per_thread_cmd;
 
 //extern guint index_threads_counter;
 extern GMutex *index_mutex;
-extern struct database *database_db;
 extern gchar *optimize_keys_str;
 extern gchar *checksum_str;
 extern gboolean no_stream;
